/**
 * Copyright (c) 2007 Colony Project
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 **/

/**
 * 
 * @file analog.h
 * @brief Contains functions and definitions for using the ADC
 * 
 * Contains definitions and function prototypes for using the
 * ADC to detect analog signals on pins AN0 - AN7.
 * AN6 and AN7 are used for the wheel and battery.
	
 * The pins labeled E6 and E7 are external interrupt pins and are not related 
 * to analog.
	
 * @author Colony Project, CMU Robotics Club, based on firefly
 * originally taken from fwr analog file (author: Tom Lauwers)
 * loop code written by Kevin Woo and James Kong
 */

#ifndef _ANALOG_H
#define _ANALOG_H

#include <inttypes.h>

/**
 * @addtogroup analog
 * @{
 **/

/** @brief Analog port 0 **/
#define AN0 0x00
/** @brief Analog port 1 **/
#define AN1 0x01
/** @brief Analog port 2 **/
#define AN2 0x02
/** @brief Analog port 3 **/
#define AN3 0x03
/** @brief Analog port 4 **/
#define AN4 0x04
/** @brief Analog port 5 **/
#define AN5 0x05
/** @brief Analog port 6 **/
#define AN6 0x06
/** @brief Analog port 7 **/
#define AN7 0x07
/** @brief Analog port 8 **/
#define AN8 0x08
/** @brief Analog port 9 **/
#define AN9 0x09
/** @brief Analog port 10 **/
#define AN10 0x0A
/** @brief Analog port 11 **/
#define AN11 0x0B
/** @brief Analog port 12 **/
#define AN12 0x0C
/** @brief Analog port 13 **/
#define AN13 0x0D
/** @brief Analog port 14 **/
#define AN14 0x0E
/** @brief Analog port 15 **/
#define AN15 0x0F

/** @brief BOM_PORT analog port for BOM **/
#define BOM_PORT AN0
/** @brief EXT_MUX analog port **/
#define EXT_MUX AN7
/** @brief Analog port for the wheel **/
#define WHEEL_PORT AN10
/** @brief Analog port for the battery voltage detector **/
#define BATT_PORT  AN11

/** @brief Analog loop status. ADC conversion running. **/
#define ADC_LOOP_RUNNING 1
/** @brief Analog loop status.  No ADC conversion running.**/
#define ADC_LOOP_STOPPED 0

/** @brief Analog init parameter. Start the analog loop. **/
#define ADC_START 1
/** @brief Analog init parameter. Don't start the analog loop. **/
#define ADC_STOP  0

#define ADMUX_OPT 0x60

/** @brief Struct to hold the value of a particular analog port **/
typedef struct {
  uint8_t adc8;
  uint16_t adc10;
} adc_t;


/** @brief Initialize analog ports. Will start running a loop
    if start_conversion is ADC_START.**/
void analog_init(int start_conversion);
/** @brief starts the analog loop. Doesn't do anything if the loop is already running. **/
void analog_start_loop(void);
/** @brief Stops the analog loop. Doesn't do anything if the loop is already stopped. **/
void analog_stop_loop(void);
/** @brief Returns the status of the analog loop. **/
int analog_loop_status(void);
/** @brief Returns an 8-bit analog value from the look up table. Use this instead of analog_get8. **/
unsigned int analog8(int which);
/** @brief Returns an 10-bit analog value from the look up table. Use this instead of analog_get10. **/
unsigned int analog10(int which);
/** @brief Read the position of the wheel. **/
int wheel(void);
/** @brief Read an 8-bit number from an analog port. Loop must be stopped for this to work. **/
unsigned int analog_get8(int which);
/** @brief Read a 10-bit number from an analog port. Loop must be stopped for this to work. **/
unsigned int analog_get10(int which);


/**@}**/ //end group

#endif

